package com.ylx.framework.web.service;

import com.ylx.common.core.domain.model.WxLoginUser;
import com.ylx.common.core.redis.RedisCache;
import com.ylx.common.utils.ServletUtils;
import com.ylx.common.utils.StringUtils;
import com.ylx.common.utils.ip.AddressUtils;
import com.ylx.common.utils.ip.IpUtils;
import com.ylx.common.utils.uuid.IdUtils;
import eu.bitwalker.useragentutils.UserAgent;

import io.jsonwebtoken.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * token验证处理
 *
 * @author ylx
 */
@Component
public class WxTokenService {
    // 令牌自定义标识
    @Value("${token.header}")
    private String header;

    // 令牌秘钥
    @Value("${token.secret}")
    private String secret;

    // 令牌有效期（默认30分钟）
    @Value("${token.expireTime}")
    private int expireTime;

    protected static final long MILLIS_SECOND = 1000;

    protected static final long MILLIS_MINUTE = 60 * MILLIS_SECOND;

    private static final Long MILLIS_MINUTE_TEN = 20 * 60 * 1000L;
    // 更换前缀的话  这个类中所有的tf: 全部更换
    private static final String wx_prefix = "tf:";

    @Autowired
    private RedisCache redisCache;

    /**
     * 获取用户身份信息
     *
     * @return 用户信息
     */
    public WxLoginUser getWxUser(HttpServletRequest request) {
        // 获取请求携带的令牌
        String token = getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            try {
                // 解析对应的权限以及用户信息
                Claims claims = parseToken(token);
                // 解析对应的权限以及用户信息
                String uuid = (String) claims.get(wx_prefix);
                String userKey = getTokenKey(uuid);
                WxLoginUser user = redisCache.getCacheObject(userKey);
                return user;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    /**
     * 设置用户身份信息
     */
    public void setWxLoginUser(WxLoginUser wxUser) {
        if (StringUtils.isNotNull(wxUser) && StringUtils.isNotEmpty(wxUser.getToken())) {
            refreshToken(wxUser);
        }
    }

    /**
     * 删除用户身份信息
     */
    public void delWxUser(String token) {
        if (StringUtils.isNotEmpty(token)) {
            String userKey = getTokenKey(token);
            redisCache.deleteObject(userKey);
        }
    }

    /**
     * 创建令牌
     *
     * @param wxUser 用户信息
     * @return 令牌
     */
    public String createToken(WxLoginUser wxUser) {
        String token = IdUtils.fastUUID();
        wxUser.setToken(token);
        setUserAgent(wxUser);
        refreshToken(wxUser);

        Map<String, Object> claims = new HashMap<>();
        claims.put(wx_prefix, token);
        return createToken(claims);
    }

    /**
     * 验证令牌有效期，相差不足20分钟，自动刷新缓存
     *
     * @param wxUser
     * @return 令牌
     */
    public void verifyToken(WxLoginUser wxUser) {
        long expireTime = wxUser.getExpireTime();
        long currentTime = System.currentTimeMillis();
        if (expireTime - currentTime <= MILLIS_MINUTE_TEN) {
            refreshToken(wxUser);
        }
    }

    /**
     * 刷新令牌有效期
     *
     * @param wxUser 登录信息
     */
    public void refreshToken(WxLoginUser wxUser) {
        wxUser.setLoginTime(System.currentTimeMillis());
        wxUser.setExpireTime(wxUser.getLoginTime() + expireTime * MILLIS_MINUTE);
        // 根据uuid将loginUser缓存
        String userKey = getTokenKey(wxUser.getToken());
        redisCache.setCacheObject(userKey, wxUser, expireTime, TimeUnit.MINUTES);
    }

    /**
     * 设置用户代理信息
     *
     * @param wxUser 登录信息
     */
    public void setUserAgent(WxLoginUser wxUser) {
        UserAgent userAgent = UserAgent.parseUserAgentString(ServletUtils.getRequest().getHeader("User-Agent"));
        String ip = IpUtils.getIpAddr();
        wxUser.setIpaddr(ip);
        wxUser.setLoginLocation(AddressUtils.getRealAddressByIP(ip));
        wxUser.setBrowser(userAgent.getBrowser().getName());
        wxUser.setOs(userAgent.getOperatingSystem().getName());
    }

    /**
     * 从数据声明生成令牌
     *
     * @param claims 数据声明
     * @return 令牌
     */
    private String createToken(Map<String, Object> claims) {
        String token = Jwts.builder()
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, secret).compact();
        return token;
    }

    /**
     * 从令牌中获取数据声明
     *
     * @param token 令牌
     * @return 数据声明
     */
    private Claims parseToken(String token) {
        JwtParser parser = Jwts.parser();
        JwtParser jwtParser = parser.setSigningKey(secret);
        Jws<Claims> claimsJws = jwtParser.parseClaimsJws(token);
        Claims body = claimsJws.getBody();
        return Jwts.parser()
                .setSigningKey(secret)
                .parseClaimsJws(token)
                .getBody();
    }

    /**
     * 从令牌中获取用户名
     *
     * @param token 令牌
     * @return 用户名
     */
    public String getUsernameFromToken(String token) {
        Claims claims = parseToken(token);
        return claims.getSubject();
    }

    /**
     * 获取请求token
     *
     * @param request
     * @return token
     */
    public String getToken(HttpServletRequest request) {
        String token = request.getHeader(header);
        if (StringUtils.isNotEmpty(token) && token.startsWith(wx_prefix)) {
            token = token.replace(wx_prefix, "");
            return token;
        } else {
            return null;
        }
    }

    private String getTokenKey(String uuid) {
        return "tf: " + uuid;
    }
}

