package com.ylx.massage.utils;
import com.ylx.common.constant.MassageConstants;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.stereotype.Component;

import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.Collections;

@Component
public class OrderNumberGenerator {

    //订单单号前缀
    public static final String KEY_PREFIX_ORDER = "YORDER";

    public static final String KEY_PREFIX_RECHAR = "RECHAR";

    public static final String KEY_PREFIX_CASH = "CASH";

    public static final String KEY_PREFIX_REFUND = "REFUND";

    private static final DateTimeFormatter DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyyMMdd");

    private final StringRedisTemplate stringRedisTemplate;

    public OrderNumberGenerator(StringRedisTemplate stringRedisTemplate) {
        this.stringRedisTemplate = stringRedisTemplate;
    }

    /**
     * 生成并返回当天的下一个自增单号。
     *
     * @return 当天的下一个单号（格式：yyyyMMdd0001）
     */
    public String generateNextOrderNumber(String prefix) {

        LocalDate today = LocalDate.now();
        String currentDateStr = today.format(DATE_FORMATTER);

        String key = MassageConstants.NUMBER + prefix + currentDateStr;
        Long incrementResult = incrementByScript(key);
        int sequence = incrementResult.intValue();
        return prefix + currentDateStr + String.format("%05d", sequence);
    }



    public Long incrementByScript(String key) {
        if (key == null || key.trim().isEmpty()) {
            throw new IllegalArgumentException("Key cannot be null or empty");
        }

        // 假设 "script" 是一个从外部文件或安全的静态资源加载的Lua脚本
        String script = "return redis.call('INCR', KEYS[1])";
        DefaultRedisScript<Long> redisScript = new DefaultRedisScript<>(script, Long.class);

        try {
            Long execute = stringRedisTemplate.execute(redisScript, Collections.singletonList(key));
            stringRedisTemplate.expireAt(key,LocalDateTime.of(LocalDate.now(), LocalTime.MAX).atZone(ZoneId.systemDefault()).toInstant());
            return execute;
        } catch (Exception e) {
            // 根据业务需求，这里可以选择抛出运行时异常或者返回特定的错误码
            throw new RuntimeException("Failed to execute Redis script", e);
        }
    }


    public static void main(String[] args) {
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime todayStart = LocalDateTime.of(LocalDate.now(), LocalTime.MIN);
        LocalDateTime todayEnd = LocalDateTime.of(LocalDate.now(), LocalTime.MAX);

        Instant instant = todayStart.atZone(ZoneId.systemDefault()).toInstant();

        System.out.println("当前时间，当天的开始时间(日期+时分秒)："+ todayStart.format(dtf));
        System.out.println("当前时间，当天的结束时间(日期+时分秒)："+ todayEnd.format(dtf));
        System.out.println("当前的时间(只有日期)"+LocalDate.now());
        System.out.println(instant);
    }



}
